/*
 * Copyright (c) 2007 Luca Barbato <lu_zero@gentoo.org>
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 ** @file libavcodec/mips/int_loongson.c
 ** integer misc ops.
 **/

#include "config.h"

#include "libavcodec/dsputil.h"

extern void vc1_inv_trans_4x4_dc_loongson(uint8_t *dest, int linesize, DCTELEM *block);
extern void vc1_inv_trans_4x8_dc_loongson(uint8_t *dest, int linesize, DCTELEM *block);
extern void vc1_inv_trans_8x4_dc_loongson(uint8_t *dest, int linesize, DCTELEM *block);
extern void vc1_inv_trans_8x8_dc_loongson(uint8_t *dest, int linesize, DCTELEM *block);
extern void vc1_inv_trans_8x8_loongson(DCTELEM *block);
extern void vc1_v_overlap_loongson(uint8_t* src, int stride);

extern void add_int16_loongson(int16_t * v1, int16_t * v2, int order);
extern void sub_int16_loongson(int16_t * v1, int16_t * v2, int order);
extern int32_t scalarproduct_int16_loongson(int16_t * v1, int16_t * v2, int order, const int shift);

extern void put_rv40_chroma_mc4_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void put_h264_chroma_mc4_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void put_rv40_chroma_mc8_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void put_h264_chroma_mc8_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void put_no_rnd_vc1_chroma_mc8_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void avg_rv40_chroma_mc4_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void avg_h264_chroma_mc4_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void avg_rv40_chroma_mc8_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void avg_h264_chroma_mc8_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);
extern void avg_no_rnd_vc1_chroma_mc8_loongson(uint8_t *dst, uint8_t *src, int srcStride, int h, int x, int y);

extern void put_qpel8_mc00_loongson(uint8_t *dst, uint8_t *src, int line_size);
extern void put_qpel16_mc00_loongson(uint8_t *dst, uint8_t *src, int line_size);

extern void put_pixels8_loongson(uint8_t *block, const uint8_t *pixels, int line_size, int h);
extern void put_pixels16_loongson(uint8_t *block, const uint8_t *pixels, int line_size, int h);
extern void add_pixels_clamped_loongson(const DCTELEM *block, uint8_t *pixels, int line_size);
extern void put_pixels_clamped_loongson(const DCTELEM *block, uint8_t *pixels, int line_size);
extern void clear_block_loongson(DCTELEM *block);
extern void clear_blocks_loongson(DCTELEM *blocks);

extern void simple_idct_loongson(DCTELEM *block);
static void simple_idct_put_loongson(uint8_t *dest, int line_size, DCTELEM *block)
{
    simple_idct_loongson(block);
    put_pixels_clamped_loongson(block, dest, line_size);
}
void simple_idct_add_loongson(uint8_t *dest, int line_size, DCTELEM *block)
{
    simple_idct_loongson(block);
    add_pixels_clamped_loongson(block, dest, line_size);
}

void dsputil_init_mips(DSPContext* c, AVCodecContext *avctx)
{
    c->vc1_inv_trans_4x4_dc = vc1_inv_trans_4x4_dc_loongson;
    c->vc1_inv_trans_4x8_dc = vc1_inv_trans_4x8_dc_loongson;
    c->vc1_inv_trans_8x4_dc = vc1_inv_trans_8x4_dc_loongson;
    c->vc1_inv_trans_8x8_dc = vc1_inv_trans_8x8_dc_loongson;
    c->vc1_inv_trans_8x8    = vc1_inv_trans_8x8_loongson;
    c->vc1_v_overlap        = vc1_v_overlap_loongson;

    c->add_int16 = add_int16_loongson;
    c->sub_int16 = sub_int16_loongson;
    c->scalarproduct_int16 = scalarproduct_int16_loongson;

    c->put_rv40_chroma_pixels_tab[0] = put_rv40_chroma_mc8_loongson;
    c->put_rv40_chroma_pixels_tab[1] = put_rv40_chroma_mc4_loongson;
    c->put_no_rnd_vc1_chroma_pixels_tab[0] = put_no_rnd_vc1_chroma_mc8_loongson;
    c->put_h264_chroma_pixels_tab[1] = put_h264_chroma_mc4_loongson;
    c->put_h264_chroma_pixels_tab[0] = put_h264_chroma_mc8_loongson;

    c->avg_rv40_chroma_pixels_tab[0] = avg_rv40_chroma_mc8_loongson;
    c->avg_rv40_chroma_pixels_tab[1] = avg_rv40_chroma_mc4_loongson;
    c->avg_no_rnd_vc1_chroma_pixels_tab[0] = avg_no_rnd_vc1_chroma_mc8_loongson;
    c->avg_h264_chroma_pixels_tab[1] = avg_h264_chroma_mc4_loongson;
    c->avg_h264_chroma_pixels_tab[0] = avg_h264_chroma_mc8_loongson;

    c->put_pixels_tab[0][0] = put_pixels16_loongson;
    c->put_pixels_tab[1][0] = put_pixels8_loongson;
    c->put_no_rnd_pixels_tab[0][0] = put_pixels16_loongson;
    c->put_no_rnd_pixels_tab[1][0] = put_pixels8_loongson;
    c->put_qpel_pixels_tab[0][0] = put_qpel16_mc00_loongson;
    c->put_qpel_pixels_tab[1][0] = put_qpel8_mc00_loongson;
    c->put_no_rnd_qpel_pixels_tab[0][0] = put_qpel16_mc00_loongson;
    c->put_no_rnd_qpel_pixels_tab[1][0] = put_qpel8_mc00_loongson;
    c->put_2tap_qpel_pixels_tab[0][0] = put_qpel16_mc00_loongson;
    c->put_2tap_qpel_pixels_tab[1][0] = put_qpel8_mc00_loongson;
    c->clear_block  = clear_block_loongson;
    c->clear_blocks = clear_blocks_loongson;
    c->add_pixels_clamped = add_pixels_clamped_loongson;

    c->idct     = simple_idct_loongson;
    c->idct_add = simple_idct_add_loongson;
    c->idct_put = simple_idct_put_loongson;
    c->idct_permutation_type = FF_SIMPLE_IDCT_PERM;
}

